// -*- C++ -*-
/*!
 * @file  ClueAnalyzer.cpp
 * @brief ClueAnalyzer Component
 * @date $Date$
 *
 * $Id$
 */

#include "ClueAnalyzer.h"

// Module specification
// <rtc-template block="module_spec">
static const char* clueanalyzer_spec[] =
  {
    "implementation_id", "ClueAnalyzer",
    "type_name",         "ClueAnalyzer",
    "description",       "ClueAnalyzer Component",
    "version",           "1.0.0",
    "vendor",            "Kenichi Ohara, AIST",
    "category",          "RTMContest",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

ClueAnalyzer::ClueAnalyzer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    // <rtc-template block="initializer">
    m_ClueDataIn("ClueData", m_ClueData),
    m_PositionDataOut("PositionData", m_PositionData),
    m_CodeDataOut("CodeData", m_CodeData),
    
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("ClueData", m_ClueDataIn);
  
  // Set OutPort buffer
  registerOutPort("PositionData", m_PositionDataOut);
  registerOutPort("CodeData", m_CodeDataOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

ClueAnalyzer::~ClueAnalyzer()
{
}



RTC::ReturnCode_t ClueAnalyzer::onInitialize()
{
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ClueAnalyzer::onFinalize()
{
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ClueAnalyzer::onActivated(RTC::UniqueId ec_id)
{
	m_PositionData.data.length(8);
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ClueAnalyzer::onDeactivated(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}

RTC::ReturnCode_t ClueAnalyzer::onExecute(RTC::UniqueId ec_id)
{
	//If Inport Data is updated, following code is executed.
	if(m_ClueDataIn.isNew()){
		//Read Inport Data
		m_ClueDataIn.read();
		//Allocate the temporary memory
		long length = m_ClueData.data.length();
		
		//Clear Temporary Buffer
		memset(TempData,0,1024);
		//Copy the Inport Data to temporary memory.
		for(unsigned int i(0);i<length;++i){
			TempData[i] = m_ClueData.data[i];
		}
		std::cout<<"CodeData Before:"<<TempData<<std::endl;

		//Split data to code data and position data
		IsClueDataSplit.SplitData(TempData);

		//Get CodeData
		std::string CodeData = IsClueDataSplit.GetCodeData();
		std::cout<<"CodeData After:"<<CodeData<<std::endl;
		m_CodeData.data.length(CodeData.length());
		for(unsigned int i(0);i< CodeData.length();++i){
			m_CodeData.data[i] = CodeData[i];
		}

		//Get PositionData
		for(int i(0);i < 4;++i){
			for(int j(0);j<2; ++j){
				m_PositionData.data[i*2+j] = IsClueDataSplit.GetPositionData(i,j);
			}
		}

		//Set code data and position data for each data port
		m_PositionDataOut.write();
		m_CodeDataOut.write();

	}
  return RTC::RTC_OK;
}

extern "C"
{
 
  void ClueAnalyzerInit(RTC::Manager* manager)
  {
    RTC::Properties profile(clueanalyzer_spec);
    manager->registerFactory(profile,
                             RTC::Create<ClueAnalyzer>,
                             RTC::Delete<ClueAnalyzer>);
  }
  
};


