// -*- C++ -*-
/*!
 * @file  ClueReader.cpp
 * @brief Clue Reader component
 * $Date$
 *
 * $Id$
 */

#include "ClueReader.h"

// Module specification
// <rtc-template block="module_spec">
static const char* cluereader_spec[] =
  {
    "implementation_id", "ClueReader",
    "type_name",         "ClueReader",
    "description",       "ClueReaderComponent",
    "version",           "0.1",
    "vendor",            "KenichiOhara,AIST",
    "category",          "RTMContest",
    "activity_type",     "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables

    ""
  };
// </rtc-template>

ClueReader::ClueReader(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_ClueDataOut("ClueData", m_ClueData)
{
   //Omajinai for OutPort
   registerOutPort("ClueData", m_ClueDataOut);

}

ClueReader::~ClueReader()
{
}


RTC::ReturnCode_t ClueReader::onInitialize()
{
   if(!qd.Initialize()){
      std::cout<<"Initialize ERROR"<<std::endl;
   }   
   return RTC::RTC_OK;
}




RTC::ReturnCode_t ClueReader::onFinalize()
{
   qd.Finalize();
   return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ClueReader::onActivated()
{
  if(!qd.QD20Initialize()){
    std::cout<<"onActivated Error"<<std::endl;
  }
   return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ClueReader::onExecute(RTC::UniqueId ec_id)
{
   char data[1024];
   int length=0;
   bzero(data,1024);
   length = qd.ReadQRCodeData(data);
   if(length>0){
     if(data[0] == 'N' && data[1] == 'G'){
       std::cout<<"Cannot Get CLUE Data!"<<std::endl;
     }
     else{
       //Allocate memory for output data
       m_ClueData.data.length(length);
       
       std::cout<<"Clue Code Length:"<<length<<" Clue Data:";
       //Copy Clue data to Outport memory
       for(int i=0;i<length;i++){
	 m_ClueData.data[i]=data[i];
	 std::cout<<data[i];
       }
       std::cout<<std::endl;
       //Output Data to OutPort
       m_ClueDataOut.write();
     }
   }
   return RTC::RTC_OK;
}

extern "C"
{
 
  void ClueReaderInit(RTC::Manager* manager)
  {
    RTC::Properties profile(cluereader_spec);
    manager->registerFactory(profile,
                             RTC::Create<ClueReader>,
                             RTC::Delete<ClueReader>);
  }
  
};


