//Serial Comuunication Class
#include"SerialComBase.h"

using namespace std;

//Constructer for this class
SerialComBase::SerialComBase(){
  fd = 0; dev = NULL;  
  bzero(&oldtio, sizeof(oldtio)); //initialize
  bzero(&newtio, sizeof(newtio)); //initialize
}

//Open the COM port
bool SerialComBase::OpenCOMDevice(const char *d){
  dev = (char*)d;
  fd = open(dev, O_RDWR | O_NOCTTY | O_NDELAY | O_NONBLOCK,0);
  if(fd < 0){
    cout<<"Cannot Open expected COM Port!"<<endl;
    return false;
  }
  else{
    cout<<"Open COM Port:"<<d<<endl;
    fcntl(fd,F_SETFL,0);

    //Flush buffered data.
    tcflush(fd, TCIOFLUSH);
    //Save old setting
    tcgetattr(fd, &oldtio);
    return true;
  }
}

//Close the COM port
bool SerialComBase::CloseCOMDevice(){
  cout<<"Close COM Port:"<<dev<<endl;
  tcsetattr(fd, TCSANOW, &oldtio);
  close(fd);
  return true;
}



//Send Message to Specified COM Device
bool SerialComBase::SendMessage(char *data, unsigned int DataLength){
  //Send Setting data
  write(fd, data, DataLength);
  return true;
}

//Receive Message from Specified COM Device
int SerialComBase::ReceiveMessage(char *buf){
  unsigned int len=0;
  //Receive data from COM Device
  len = read(fd, buf,1024);
  return (int)len;
}

//Set the Communication Speed 
void SerialComBase::SetBaudRate(unsigned int speed){
  //4800bps
  if(speed==4800){ 
    cfsetispeed(&newtio,B4800);
    cfsetospeed(&newtio,B4800);
    BaudRate=4800;
  }
  //9600bps
  else if(speed==9600){ 
    cfsetispeed(&newtio,B9600);
    cfsetospeed(&newtio,B9600);
    BaudRate=9600;
  }
  //19200bps
  else if(speed==19200){ 
    cfsetispeed(&newtio,B19200);
    cfsetospeed(&newtio,B19200);
    BaudRate=19200;
  }
  //38400bps
  else if(speed==38400){ 
    cfsetispeed(&newtio,B38400);
    cfsetospeed(&newtio,B38400);
    BaudRate=38400;
  }
  //57600bps
  else if(speed==57600){ 
    cfsetispeed(&newtio, B57600);
    cfsetospeed(&newtio, B57600);
    BaudRate = 57600;
  }
  //115200bps
  else if(speed==115200){ 
    cfsetispeed(&newtio,B115200);
    cfsetospeed(&newtio,B115200);
    BaudRate=115200;
  }
  else { //use standard speed
    cfsetispeed(&newtio, B9600);
    cfsetospeed(&newtio, B9600);    
    BaudRate = 9600;
  }
  tcsetattr(fd, TCSANOW, &newtio);
}

//Get Communication Speed
unsigned int SerialComBase::GetBaudRate(){
  return BaudRate;
}

//Set Data bit(5,6,7or8) 
void SerialComBase::SetDataBit(int bit){
  switch(bit){
  case 5:
    newtio.c_cflag |= CS5;
    break;
  case 6:
    newtio.c_cflag |= CS6;
    break;
  case 7:
    newtio.c_cflag |= CS7;
    break;
  case 8:
    newtio.c_cflag &= ~CSIZE;
    newtio.c_cflag |= CS8;
    break;
  default:
    newtio.c_cflag &= ~CSIZE;
    newtio.c_cflag |= CS8;
    break;
  }
  DataBit = bit;
  tcsetattr(fd, TCSANOW, &newtio);
}

//Get Data Bit
int SerialComBase::GetDataBit(){
  return DataBit;
}

//Set '2' for stop bit
void SerialComBase::SetStopBit2(){
  newtio.c_cflag |= CSTOPB;
  tcsetattr(fd, TCSANOW, &newtio);
}

//Change Stop Bit '1'
void SerialComBase::ClearStopBit2(){
  newtio.c_cflag &= ~CSTOPB; 
  tcsetattr(fd, TCSANOW, &newtio);  
}

//Set the parameter about parity check
void SerialComBase::SetParityFlag(unsigned int parityFlag){
  if(parityFlag == NOPARITY){
    newtio.c_iflag |= IGNPAR;
  }
  else if(parityFlag == ODDPARITY){
    newtio.c_iflag |= INPCK;  //do the parity check about received data  
    newtio.c_cflag |= PARENB; //do parity check
    newtio.c_cflag |= PARODD; //When you set this option, odd parity is used to parity check.And, if you don't set this bit, even parity is used to parity check.
  }
  else if(parityFlag == EVENPARITY){
    newtio.c_iflag |= INPCK;  //do the parity check about received data    
    newtio.c_cflag |= PARENB; //do parity check
  }
  tcsetattr(fd, TCSANOW, &newtio);
}


void SerialComBase::EnableRead(){
  newtio.c_cflag |= CREAD;
  tcsetattr(fd, TCSANOW, &newtio);
}

void SerialComBase::DisableRead(){
  newtio.c_cflag &= ~CREAD;
  tcsetattr(fd, TCSANOW, &newtio);
}

//Receive mode setting(sync or asynch)
void SerialComBase::EnableCanonical(){
  newtio.c_lflag |= ICANON;
  tcsetattr(fd, TCSANOW, &newtio);
}

void SerialComBase::DisableCanonical(){
  newtio.c_lflag &= ~ICANON;
  tcsetattr(fd, TCSANOW, &newtio);
}

//Set Receive Delimiter Type
void SerialComBase::SetReceiveDelimiter(int mode){
  switch(mode){
  case CRNL:
    newtio.c_iflag |= ICRNL;    
    break;
  case NLCR:
    newtio.c_iflag |= INLCR;
    break;
  case NOTHING:
    newtio.c_iflag |= IGNCR;    
    break;
  default:
    newtio.c_iflag |= ICRNL;    
    break;
  }
  Delimiter = mode;
  tcsetattr(fd, TCSANOW, &newtio);
}

//Get Delimiter Type
int SerialComBase::GetReceiveDelimiter(){
  return Delimiter;
}

//Enable local connect. when you want to control the Modem,please call DisableLocalConnect
void SerialComBase::EnableLocalConnect(){
  newtio.c_cflag |= CLOCAL;
  tcsetattr(fd, TCSANOW, &newtio);
  
}

void SerialComBase::DisableLocalConnect(){
  newtio.c_cflag &= ~CLOCAL;
  tcsetattr(fd, TCSANOW, &newtio);
}


